@@ -30,6 +30,7 @@
 #include "libsmb/libsmb.h"
 #include "source3/include/messages.h"
 #include "source3/include/g_lock.h"
+#include "lib/util/util_tdb.h"
 
 /*********************************************************
  Change the domain password on the PDC.
@@ -44,7 +45,8 @@ struct trust_pw_change_state {
 
 static int trust_pw_change_state_destructor(struct trust_pw_change_state *state)
 {
-	g_lock_unlock(state->g_ctx, state->g_lock_key);
+	g_lock_unlock(state->g_ctx,
+		      string_term_tdb_data(state->g_lock_key));
 	return 0;
 }
 
@@ -82,7 +84,6 @@ char *trust_pw_new_value(TALLOC_CTX *mem_ctx,
 		min = 120;
 		max = 120;
 		break;
-		/* fall through */
 	case SEC_CHAN_DOMAIN:
 		/*
 		 * The maximum length of a trust account password.
@@ -104,6 +105,36 @@ char *trust_pw_new_value(TALLOC_CTX *mem_ctx,
 	return generate_random_machine_password(mem_ctx, min, max);
 }
 
+/*
+ * Temporary function to wrap cli_auth in a lck
+ */
+
+static NTSTATUS netlogon_creds_cli_lck_auth(
+	struct netlogon_creds_cli_context *context,
+	struct dcerpc_binding_handle *b,
+	uint8_t num_nt_hashes,
+	const struct samr_Password * const *nt_hashes,
+	uint8_t *idx_nt_hashes)
+{
+	struct netlogon_creds_cli_lck *lck;
+	NTSTATUS status;
+
+	status = netlogon_creds_cli_lck(
+		context, NETLOGON_CREDS_CLI_LCK_EXCLUSIVE,
+		talloc_tos(), &lck);
+	if (!NT_STATUS_IS_OK(status)) {
+		DBG_WARNING("netlogon_creds_cli_lck failed: %s\n",
+			    nt_errstr(status));
+		return status;
+	}
+
+	status = netlogon_creds_cli_auth(context, b, num_nt_hashes, nt_hashes,
+					 idx_nt_hashes);
+	TALLOC_FREE(lck);
+
+	return status;
+}
+
 NTSTATUS trust_pw_change(struct netlogon_creds_cli_context *context,
 			 struct messaging_context *msg_ctx,
 			 struct dcerpc_binding_handle *b,
@@ -161,7 +192,7 @@ NTSTATUS trust_pw_change(struct netlogon_creds_cli_context *context,
 
 	g_timeout = timeval_current_ofs(10, 0);
 	status = g_lock_lock(state->g_ctx,
-			     state->g_lock_key,
+			     string_term_tdb_data(state->g_lock_key),
 			     G_LOCK_WRITE, g_timeout);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(1, ("could not get g_lock on [%s]!\n",
@@ -323,7 +354,7 @@ NTSTATUS trust_pw_change(struct netlogon_creds_cli_context *context,
 		}
 
 		/*
-		 * We use the password that's already persitent in
+		 * We use the password that's already persistent in
 		 * our database in order to handle failures.
 		 */
 		data_blob_clear_free(&new_trust_pw_blob);
@@ -358,10 +389,10 @@ NTSTATUS trust_pw_change(struct netlogon_creds_cli_context *context,
 	 * ServerTrustPasswordsGet() or netr_ServerGetTrustInfo() to fix our
 	 * local secrets before doing the change.
 	 */
-	status = netlogon_creds_cli_auth(context, b,
-					 num_nt_hashes,
-					 nt_hashes,
-					 &idx_nt_hashes);
+	status = netlogon_creds_cli_lck_auth(context, b,
+					     num_nt_hashes,
+					     nt_hashes,
+					     &idx_nt_hashes);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("netlogon_creds_cli_auth(%s) failed for old passwords (%u) - %s!\n",
 			  context_name, num_nt_hashes, nt_errstr(status)));
@@ -568,13 +599,12 @@ NTSTATUS trust_pw_change(struct netlogon_creds_cli_context *context,
 	 * Now we verify the new password.
 	 */
 	idx = 0;
-	idx_current = idx;
 	nt_hashes[idx++] = current_nt_hash;
 	num_nt_hashes = idx;
-	status = netlogon_creds_cli_auth(context, b,
-					 num_nt_hashes,
-					 nt_hashes,
-					 &idx_nt_hashes);
+	status = netlogon_creds_cli_lck_auth(context, b,
+					     num_nt_hashes,
+					     nt_hashes,
+					     &idx_nt_hashes);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("netlogon_creds_cli_auth(%s) failed for new password - %s!\n",
 			  context_name, nt_errstr(status)));
