@@ -30,11 +30,13 @@
 #include "system/filesys.h"
 #include "smbd/smbd.h"
 #include "libcli/security/security_token.h"
+#include "libcli/security/dom_sid.h"
 #include "nfs4_acls.h"
 #include "librpc/gen_ndr/ndr_nfs4acl.h"
 #include "nfs4acl_xattr.h"
 #include "nfs4acl_xattr_ndr.h"
 #include "nfs4acl_xattr_xdr.h"
+#include "nfs4acl_xattr_nfs.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_VFS
@@ -42,6 +44,7 @@
 static const struct enum_list nfs4acl_encoding[] = {
 	{NFS4ACL_ENCODING_NDR, "ndr"},
 	{NFS4ACL_ENCODING_XDR, "xdr"},
+	{NFS4ACL_ENCODING_NFS, "nfs"},
 };
 
 /*
@@ -53,13 +56,16 @@ static bool nfs4acl_validate_blob(vfs_handle_struct *handle,
 {
 	struct nfs4acl_config *config = NULL;
 	mode_t expected_mode;
-	int saved_errno = 0;
 	int ret;
 
 	SMB_VFS_HANDLE_GET_DATA(handle, config,
 				struct nfs4acl_config,
 				return false);
 
+	if (!config->validate_mode) {
+		return true;
+	}
+
 	if (!VALID_STAT(smb_fname->st)) {
 		/* might be a create */
 		return true;
@@ -74,17 +80,9 @@ static bool nfs4acl_validate_blob(vfs_handle_struct *handle,
 		return true;
 	}
 
-	become_root();
 	ret = SMB_VFS_NEXT_REMOVEXATTR(handle,
 				       smb_fname,
 				       config->xattr_name);
-	if (ret != 0) {
-		saved_errno = errno;
-	}
-	unbecome_root();
-	if (saved_errno != 0) {
-		errno = saved_errno;
-	}
 	if (ret != 0 && errno != ENOATTR) {
 		DBG_ERR("Removing NFS4 xattr failed: %s\n", strerror(errno));
 		return false;
@@ -128,7 +126,6 @@ static NTSTATUS nfs4acl_get_blob(struct vfs_handle_struct *handle,
 	}
 
 	do {
-		int saved_errno = 0;
 
 		allocsize *= 4;
 		ok = data_blob_realloc(mem_ctx, blob, allocsize);
@@ -136,7 +133,6 @@ static NTSTATUS nfs4acl_get_blob(struct vfs_handle_struct *handle,
 			return NT_STATUS_NO_MEMORY;
 		}
 
-		become_root();
 		if (fsp != NULL && fsp->fh->fd != -1) {
 			length = SMB_VFS_NEXT_FGETXATTR(handle,
 							fsp,
@@ -150,13 +146,6 @@ static NTSTATUS nfs4acl_get_blob(struct vfs_handle_struct *handle,
 						       blob->data,
 						       blob->length);
 		}
-		if (length == -1) {
-			saved_errno = errno;
-		}
-		unbecome_root();
-		if (saved_errno != 0) {
-			errno = saved_errno;
-		}
 	} while (length == -1 && errno == ERANGE && allocsize <= 65536);
 
 	if (length == -1) {
@@ -228,6 +217,9 @@ static NTSTATUS nfs4acl_blob_to_smb4(struct vfs_handle_struct *handle,
 	case NFS4ACL_ENCODING_XDR:
 		status = nfs4acl_xdr_blob_to_smb4(handle, mem_ctx, blob, smb4acl);
 		break;
+	case NFS4ACL_ENCODING_NFS:
+		status = nfs4acl_nfs_blob_to_smb4(handle, mem_ctx, blob, smb4acl);
+		break;
 	default:
 		status = NT_STATUS_INTERNAL_ERROR;
 		break;
@@ -270,17 +262,20 @@ static NTSTATUS nfs4acl_xattr_fget_nt_acl(struct vfs_handle_struct *handle,
 	return status;
 }
 
-static NTSTATUS nfs4acl_xattr_get_nt_acl(struct vfs_handle_struct *handle,
-				  const struct smb_filename *smb_fname,
-				  uint32_t security_info,
-				  TALLOC_CTX *mem_ctx,
-				  struct security_descriptor **sd)
+static NTSTATUS nfs4acl_xattr_get_nt_acl_at(struct vfs_handle_struct *handle,
+				struct files_struct *dirfsp,
+				const struct smb_filename *smb_fname,
+				uint32_t security_info,
+				TALLOC_CTX *mem_ctx,
+				struct security_descriptor **sd)
 {
 	struct SMB4ACL_T *smb4acl = NULL;
 	TALLOC_CTX *frame = talloc_stackframe();
 	DATA_BLOB blob;
 	NTSTATUS status;
 
+	SMB_ASSERT(dirfsp == handle->conn->cwd_fsp);
+
 	status = nfs4acl_get_blob(handle, NULL, smb_fname, frame, &blob);
 	if (NT_STATUS_EQUAL(status, NT_STATUS_NOT_FOUND)) {
 		TALLOC_FREE(frame);
@@ -328,6 +323,10 @@ static bool nfs4acl_smb4acl_set_fn(vfs_handle_struct *handle,
 		status = nfs4acl_smb4acl_to_xdr_blob(handle, talloc_tos(),
 						     smb4acl, &blob);
 		break;
+	case NFS4ACL_ENCODING_NFS:
+		status = nfs4acl_smb4acl_to_nfs_blob(handle, talloc_tos(),
+						     smb4acl, &blob);
+		break;
 	default:
 		status = NT_STATUS_INTERNAL_ERROR;
 		break;
@@ -336,7 +335,6 @@ static bool nfs4acl_smb4acl_set_fn(vfs_handle_struct *handle,
 		return false;
 	}
 
-	become_root();
 	if (fsp->fh->fd != -1) {
 		ret = SMB_VFS_NEXT_FSETXATTR(handle, fsp, config->xattr_name,
 					     blob.data, blob.length, 0);
@@ -348,7 +346,6 @@ static bool nfs4acl_smb4acl_set_fn(vfs_handle_struct *handle,
 	if (ret != 0) {
 		saved_errno = errno;
 	}
-	unbecome_root();
 	data_blob_free(&blob);
 	if (saved_errno != 0) {
 		errno = saved_errno;
@@ -372,6 +369,7 @@ static NTSTATUS nfs4acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 	mode_t expected_mode;
 	mode_t restored_mode;
 	bool chown_needed = false;
+	struct dom_sid_buf buf;
 	NTSTATUS status;
 	int ret;
 
@@ -390,12 +388,14 @@ static NTSTATUS nfs4acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 	} else {
 		expected_mode = 0666;
 	}
+	if (!config->validate_mode) {
+		existing_mode = 0;
+		expected_mode = 0;
+	}
 	if ((existing_mode & expected_mode) != expected_mode) {
-		int saved_errno = 0;
 
 		restored_mode = existing_mode | expected_mode;
 
-		become_root();
 		if (fsp->fh->fd != -1) {
 			ret = SMB_VFS_NEXT_FCHMOD(handle,
 						  fsp,
@@ -405,13 +405,6 @@ static NTSTATUS nfs4acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 						 fsp->fsp_name,
 						 restored_mode);
 		}
-		if (ret != 0) {
-			saved_errno = errno;
-		}
-		unbecome_root();
-		if (saved_errno != 0) {
-			errno = saved_errno;
-		}
 		if (ret != 0) {
 			DBG_ERR("Resetting POSIX mode on [%s] from [0%o]: %s\n",
 				fsp_str_dbg(fsp), existing_mode,
@@ -429,6 +422,9 @@ static NTSTATUS nfs4acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 	if (NT_STATUS_IS_OK(status)) {
 		return NT_STATUS_OK;
 	}
+	if (!NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED)) {
+		return status;
+	}
 
 	/*
 	 * We got access denied. If we're already root, or we didn't
@@ -467,16 +463,15 @@ static NTSTATUS nfs4acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 	}
 
 	DBG_DEBUG("overriding chown on file %s for sid %s\n",
-		  fsp_str_dbg(fsp), sid_string_tos(psd->owner_sid));
+		  fsp_str_dbg(fsp),
+		  dom_sid_str_buf(psd->owner_sid, &buf));
 
-	become_root();
 	status = smb_set_nt_acl_nfs4(handle,
 				     fsp,
 				     &config->nfs4_params,
 				     security_info_sent,
 				     psd,
 				     nfs4acl_smb4acl_set_fn);
-	unbecome_root();
 	return status;
 }
 
@@ -484,9 +479,12 @@ static int nfs4acl_connect(struct vfs_handle_struct *handle,
 			   const char *service,
 			   const char *user)
 {
+	const struct loadparm_substitution *lp_sub =
+		loadparm_s3_global_substitution();
 	struct nfs4acl_config *config = NULL;
 	const struct enum_list *default_acl_style_list = NULL;
 	const char *default_xattr_name = NULL;
+	bool default_validate_mode = true;
 	int enumval;
 	unsigned nfs_version;
 	int ret;
@@ -526,6 +524,10 @@ static int nfs4acl_connect(struct vfs_handle_struct *handle,
 	case NFS4ACL_ENCODING_XDR:
 		default_xattr_name = NFS4ACL_XDR_XATTR_NAME;
 		break;
+	case NFS4ACL_ENCODING_NFS:
+		default_xattr_name = NFS4ACL_NFS_XATTR_NAME;
+		default_validate_mode = false;
+		break;
 	case NFS4ACL_ENCODING_NDR:
 	default:
 		default_xattr_name = NFS4ACL_NDR_XATTR_NAME;
@@ -554,12 +556,23 @@ static int nfs4acl_connect(struct vfs_handle_struct *handle,
 						 default_acl_style_list,
 						 DEFAULT_ACL_EVERYONE);
 
-	config->xattr_name = lp_parm_talloc_string(config,
+	config->xattr_name = lp_parm_substituted_string(config, lp_sub,
 						   SNUM(handle->conn),
 						   "nfs4acl_xattr",
 						   "xattr_name",
 						   default_xattr_name);
 
+	config->nfs4_id_numeric = lp_parm_bool(SNUM(handle->conn),
+					       "nfs4acl_xattr",
+					       "nfs4_id_numeric",
+					       false);
+
+
+	config->validate_mode = lp_parm_bool(SNUM(handle->conn),
+					     "nfs4acl_xattr",
+					     "validate_mode",
+					     default_validate_mode);
+
 	SMB_VFS_HANDLE_SET_DATA(handle, config, NULL, struct nfs4acl_config,
 				return -1);
 
@@ -647,7 +660,7 @@ static int nfs4acl_xattr_fail__sys_acl_blob_get_fd(vfs_handle_struct *handle, fi
 static struct vfs_fn_pointers nfs4acl_xattr_fns = {
 	.connect_fn = nfs4acl_connect,
 	.fget_nt_acl_fn = nfs4acl_xattr_fget_nt_acl,
-	.get_nt_acl_fn = nfs4acl_xattr_get_nt_acl,
+	.get_nt_acl_at_fn = nfs4acl_xattr_get_nt_acl_at,
 	.fset_nt_acl_fn = nfs4acl_xattr_fset_nt_acl,
 
 	.sys_acl_get_file_fn = nfs4acl_xattr_fail__sys_acl_get_file,
