@@ -34,12 +34,10 @@
 #include "util_sd.h"
 
 static int test_args;
-
-#define CREATE_ACCESS_READ READ_CONTROL_ACCESS
-
 static int sddl;
 static int query_sec_info = -1;
 static int set_sec_info = -1;
+static bool want_mxac;
 
 static const char *domain_sid = NULL;
 
@@ -104,6 +102,7 @@ done:
 static struct dom_sid *get_domain_sid(struct cli_state *cli)
 {
 	NTSTATUS status;
+	struct dom_sid_buf buf;
 
 	struct dom_sid *sid = talloc(talloc_tos(), struct dom_sid);
 	if (sid == NULL) {
@@ -126,7 +125,7 @@ static struct dom_sid *get_domain_sid(struct cli_state *cli)
 
 	}
 
-	DEBUG(2,("Domain SID: %s\n", sid_string_dbg(sid)));
+	DEBUG(2,("Domain SID: %s\n", dom_sid_str_buf(sid, &buf)));
 	return sid;
 }
 
@@ -235,9 +234,19 @@ static uint16_t get_fileinfo(struct cli_state *cli, const char *filename)
 	/* The desired access below is the only one I could find that works
 	   with NT4, W2KP and Samba */
 
-	status = cli_ntcreate(cli, filename, 0, CREATE_ACCESS_READ,
-			      0, FILE_SHARE_READ|FILE_SHARE_WRITE,
-			      FILE_OPEN, 0x0, 0x0, &fnum, &cr);
+	status = cli_ntcreate(
+		cli,			/* cli */
+		filename,		/* fname */
+		0,			/* CreatFlags */
+		READ_CONTROL_ACCESS,	/* CreatFlags */
+		0,			/* FileAttributes */
+		FILE_SHARE_READ|
+		FILE_SHARE_WRITE,	/* ShareAccess */
+		FILE_OPEN,		/* CreateDisposition */
+		0x0,			/* CreateOptions */
+		0x0,			/* SecurityFlags */
+		&fnum,			/* pfid */
+		&cr);			/* cr */
 	if (!NT_STATUS_IS_OK(status)) {
 		printf("Failed to open %s: %s\n", filename, nt_errstr(status));
 		return 0;
@@ -358,12 +367,33 @@ static bool set_secdesc(struct cli_state *cli, const char *filename,
 	return result;
 }
 
+/*****************************************************
+get maximum access for a file
+*******************************************************/
+static int cacl_mxac(struct cli_state *cli, const char *filename)
+{
+	NTSTATUS status;
+	uint32_t mxac;
+
+	status = cli_query_mxac(cli, filename, &mxac);
+	if (!NT_STATUS_IS_OK(status)) {
+		printf("Failed to get mxac: %s\n", nt_errstr(status));
+		return EXIT_FAILED;
+	}
+
+	printf("Maximum access: 0x%x\n", mxac);
+
+	return EXIT_OK;
+}
+
+
 /*****************************************************
 dump the acls for a file
 *******************************************************/
 static int cacl_dump(struct cli_state *cli, const char *filename, bool numeric)
 {
 	struct security_descriptor *sd;
+	int ret;
 
 	if (test_args) {
 		return EXIT_OK;
@@ -385,6 +415,13 @@ static int cacl_dump(struct cli_state *cli, const char *filename, bool numeric)
 		sec_desc_print(cli, stdout, sd, numeric);
 	}
 
+	if (want_mxac) {
+		ret = cacl_mxac(cli, filename);
+		if (ret != EXIT_OK) {
+			return ret;
+		}
+	}
+
 	return EXIT_OK;
 }
 
@@ -469,10 +506,8 @@ static void sort_acl(struct security_acl *the_acl)
 	for (i=1;i<the_acl->num_aces;) {
 		if (security_ace_equal(&the_acl->aces[i-1],
 				       &the_acl->aces[i])) {
-			int j;
-			for (j=i; j<the_acl->num_aces-1; j++) {
-				the_acl->aces[j] = the_acl->aces[j+1];
-			}
+			ARRAY_DEL_ELEMENT(
+				the_acl->aces, i, the_acl->num_aces);
 			the_acl->num_aces--;
 		} else {
 			i++;
@@ -629,7 +664,7 @@ static int inherit(struct cli_state *cli, const char *filename,
 	if (strcmp(type,"allow")==0) {
 		if ((old->type & SEC_DESC_DACL_PROTECTED) ==
                     SEC_DESC_DACL_PROTECTED) {
-			int i;
+			uint32_t i;
 			char *parentname,*temp;
 			struct security_descriptor *parent;
 			temp = talloc_strdup(talloc_tos(), filename);
@@ -697,9 +732,12 @@ static int inherit(struct cli_state *cli, const char *filename,
                     SEC_DESC_DACL_PROTECTED) {
 			old->type=old->type | SEC_DESC_DACL_PROTECTED;
 
-			/* convert all inherited ACL's to non inherated ACL's. */
+			/*
+			 * convert all inherited ACL's to non
+			 * inherited ACL's.
+			 */
 			if (old->dacl) {
-				int i;
+				uint32_t i;
 				for (i=0;i<old->dacl->num_aces;i++) {
 					struct security_ace *ace=&old->dacl->aces[i];
 					/* Remove INHERITED FLAG from all aces */
@@ -736,17 +774,10 @@ static struct cli_state *connect_one(const struct user_auth_info *auth_info,
 	NTSTATUS nt_status;
 	uint32_t flags = 0;
 
-	if (get_cmdline_auth_info_use_kerberos(auth_info)) {
-		flags |= CLI_FULL_CONNECTION_USE_KERBEROS |
-			 CLI_FULL_CONNECTION_FALLBACK_AFTER_KERBEROS;
-	}
-
-	nt_status = cli_full_connection(&c, lp_netbios_name(), server,
+	nt_status = cli_full_connection_creds(&c, lp_netbios_name(), server,
 				NULL, 0,
 				share, "?????",
-				get_cmdline_auth_info_username(auth_info),
-				lp_workgroup(),
-				get_cmdline_auth_info_password(auth_info),
+				get_cmdline_auth_info_creds(auth_info),
 				flags,
 				get_cmdline_auth_info_signing_state(auth_info));
 	if (!NT_STATUS_IS_OK(nt_status)) {
@@ -755,10 +786,8 @@ static struct cli_state *connect_one(const struct user_auth_info *auth_info,
 	}
 
 	if (get_cmdline_auth_info_smb_encrypt(auth_info)) {
-		nt_status = cli_cm_force_encryption(c,
-					get_cmdline_auth_info_username(auth_info),
-					get_cmdline_auth_info_password(auth_info),
-					lp_workgroup(),
+		nt_status = cli_cm_force_encryption_creds(c,
+					get_cmdline_auth_info_creds(auth_info),
 					share);
                 if (!NT_STATUS_IS_OK(nt_status)) {
 			cli_shutdown(c);
@@ -787,27 +816,139 @@ int main(int argc, char *argv[])
 	/* numeric is set when the user wants numeric SIDs and ACEs rather
 	   than going via LSA calls to resolve them */
 	int numeric = 0;
+	struct cli_state *targetcli = NULL;
+	char *targetfile = NULL;
+	NTSTATUS status;
 
 	struct poptOption long_options[] = {
 		POPT_AUTOHELP
-		{ "delete", 'D', POPT_ARG_STRING, NULL, 'D', "Delete an acl", "ACL" },
-		{ "modify", 'M', POPT_ARG_STRING, NULL, 'M', "Modify an acl", "ACL" },
-		{ "add", 'a', POPT_ARG_STRING, NULL, 'a', "Add an acl", "ACL" },
-		{ "set", 'S', POPT_ARG_STRING, NULL, 'S', "Set acls", "ACLS" },
-		{ "chown", 'C', POPT_ARG_STRING, NULL, 'C', "Change ownership of a file", "USERNAME" },
-		{ "chgrp", 'G', POPT_ARG_STRING, NULL, 'G', "Change group ownership of a file", "GROUPNAME" },
-		{ "inherit", 'I', POPT_ARG_STRING, NULL, 'I', "Inherit allow|remove|copy" },
-		{ "numeric", 0, POPT_ARG_NONE, &numeric, 1, "Don't resolve sids or masks to names" },
-		{ "sddl", 0, POPT_ARG_NONE, &sddl, 1, "Output and input acls in sddl format" },
-		{ "query-security-info", 0, POPT_ARG_INT, &query_sec_info, 1,
-		  "The security-info flags for queries"
+		{
+			.longName   = "delete",
+			.shortName  = 'D',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'D',
+			.descrip    = "Delete an acl",
+			.argDescrip = "ACL",
 		},
-		{ "set-security-info", 0, POPT_ARG_INT, &set_sec_info, 1,
-		  "The security-info flags for modifications"
+		{
+			.longName   = "modify",
+			.shortName  = 'M',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'M',
+			.descrip    = "Modify an acl",
+			.argDescrip = "ACL",
+		},
+		{
+			.longName   = "add",
+			.shortName  = 'a',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'a',
+			.descrip    = "Add an acl",
+			.argDescrip = "ACL",
+		},
+		{
+			.longName   = "set",
+			.shortName  = 'S',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'S',
+			.descrip    = "Set acls",
+			.argDescrip = "ACLS",
+		},
+		{
+			.longName   = "chown",
+			.shortName  = 'C',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'C',
+			.descrip    = "Change ownership of a file",
+			.argDescrip = "USERNAME",
+		},
+		{
+			.longName   = "chgrp",
+			.shortName  = 'G',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'G',
+			.descrip    = "Change group ownership of a file",
+			.argDescrip = "GROUPNAME",
+		},
+		{
+			.longName   = "inherit",
+			.shortName  = 'I',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'I',
+			.descrip    = "Inherit allow|remove|copy",
+		},
+		{
+			.longName   = "numeric",
+			.shortName  = 0,
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = &numeric,
+			.val        = 1,
+			.descrip    = "Don't resolve sids or masks to names",
+		},
+		{
+			.longName   = "sddl",
+			.shortName  = 0,
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = &sddl,
+			.val        = 1,
+			.descrip    = "Output and input acls in sddl format",
+		},
+		{
+			.longName   = "query-security-info",
+			.shortName  = 0,
+			.argInfo    = POPT_ARG_INT,
+			.arg        = &query_sec_info,
+			.val        = 1,
+			.descrip    = "The security-info flags for queries"
+		},
+		{
+			.longName   = "set-security-info",
+			.shortName  = 0,
+			.argInfo    = POPT_ARG_INT,
+			.arg        = &set_sec_info,
+			.val        = 1,
+			.descrip    = "The security-info flags for modifications"
+		},
+		{
+			.longName   = "test-args",
+			.shortName  = 't',
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = &test_args,
+			.val        = 1,
+			.descrip    = "Test arguments"
+		},
+		{
+			.longName   = "domain-sid",
+			.shortName  = 0,
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = &domain_sid,
+			.val        = 0,
+			.descrip    = "Domain SID for sddl",
+			.argDescrip = "SID"},
+		{
+			.longName   = "max-protocol",
+			.shortName  = 'm',
+			.argInfo    = POPT_ARG_STRING,
+			.arg        = NULL,
+			.val        = 'm',
+			.descrip    = "Set the max protocol level",
+			.argDescrip = "LEVEL",
+		},
+		{
+			.longName   = "maximum-access",
+			.shortName  = 'x',
+			.argInfo    = POPT_ARG_NONE,
+			.arg        = NULL,
+			.val        = 'x',
+			.descrip    = "Query maximum persmissions",
 		},
-		{ "test-args", 't', POPT_ARG_NONE, &test_args, 1, "Test arguments"},
-		{ "domain-sid", 0, POPT_ARG_STRING, &domain_sid, 0, "Domain SID for sddl", "SID"},
-		{ "max-protocol", 'm', POPT_ARG_STRING, NULL, 'm', "Set the max protocol level", "LEVEL" },
 		POPT_COMMON_SAMBA
 		POPT_COMMON_CONNECTION
 		POPT_COMMON_CREDENTIALS
@@ -873,6 +1014,9 @@ int main(int argc, char *argv[])
 		case 'm':
 			lp_set_cmdline("client max protocol", poptGetOptArg(pc));
 			break;
+		case 'x':
+			want_mxac = true;
+			break;
 		}
 	}
 
@@ -908,8 +1052,8 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 	share = strchr_m(server,'\\');
-	if (!share) {
-		printf("Invalid argument: %s\n", share);
+	if (share == NULL) {
+		printf("Invalid argument\n");
 		return -1;
 	}
 
@@ -936,16 +1080,28 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	status = cli_resolve_path(frame,
+				  "",
+				  popt_get_cmdline_auth_info(),
+				  cli,
+				  filename,
+				  &targetcli,
+				  &targetfile);
+	if (!NT_STATUS_IS_OK(status)) {
+		DEBUG(0,("cli_resolve_path failed for %s! (%s)\n", filename, nt_errstr(status)));
+		return -1;
+	}
+
 	/* Perform requested action */
 
 	if (change_mode == REQUEST_INHERIT) {
-		result = inherit(cli, filename, owner_username);
+		result = inherit(targetcli, targetfile, owner_username);
 	} else if (change_mode != REQUEST_NONE) {
-		result = owner_set(cli, change_mode, filename, owner_username);
+		result = owner_set(targetcli, change_mode, targetfile, owner_username);
 	} else if (the_acl) {
-		result = cacl_set(cli, filename, the_acl, mode, numeric);
+		result = cacl_set(targetcli, targetfile, the_acl, mode, numeric);
 	} else {
-		result = cacl_dump(cli, filename, numeric);
+		result = cacl_dump(targetcli, targetfile, numeric);
 	}
 
 	popt_free_cmdline_auth_info();
